# SmartLogChecker
#
# This is a Nagios plugin script which greps a log file for a set of
# regexps to match. If any of them match, the script generates a Nagios
# alert. You can also supply a list of regexps to ignore. Lines
# matching an ignore expression will be ignored.

# An example use case for this might be the situation where you want
# to look for 'error', but not match text like 'No errors found'.

# Written by John Arundel
# Copyright 2007 Rivergold Associates

require 'optparse'

class SmartLogChecker

    def initialize( filename )
        @filename = filename
    end

    def find( queries, ignores = [] )
        search_pattern = [queries].join( '|' )
        ignore_pattern = [ignores].join( '|' )
        File.open( @filename ).select do |line|
            if ignore_pattern.empty?
                line =~ /#{search_pattern}/i
            else
                line =~ /#{search_pattern}/i && 
                line !~ /#{ignore_pattern}/i
            end
        end
    end
end

# Main script

OK = 0
CRITICAL_ERROR = 2

$checker = nil
$queries = nil
$ignores = nil

OptionParser.new do |opts|
  opts.banner = "Usage: #{$0} [options]\n \
\n\
Checks a log file for a specified regular expression (optionally ignoring lines matching another expression)"

  opts.separator ""
  opts.separator "Specific options:"

  opts.on("-F FILE", "File to check") do |filename|
    $checker = SmartLogChecker.new( filename )
  end

  opts.on("-q QUERY", "Regular expression to match") do |query|
    $queries = query.split( ',' )
  end

  opts.on("-i IGNORE", "Regular expression to ignore") do |ignore|
    $ignores = ignore.split( ',' )
  end

  opts.on("-h", "Show usage") do 
    puts opts
    exit(0)
  end

end.parse!

unless $checker.nil?
    results = $checker.find( $queries, $ignores )
    if results.length > 0
        p "(#{results.length}) #{results.last}"
        exit CRITICAL_ERROR
    else
        p "Log check ok - 0 pattern matches found"
        exit OK
    end
end

